function [l r] = datainfo(S)
% function [l d] = datainfo(S)
% Computes the Minimum Local Feature Size, l and
% the maximum distance between neighbouring points r, which is an 
% approximation for the maximum distance between two connected data points

l = inf; % smallest distance between two points
r = 0;   % largest distance between two neighbouring points
n = length(S);

for i = 1:n-1
  % find the largest ball around S(:,i) that doesn't contain another point in S
  s = inf; % radius of the largest such ball
  for j = i+1:n
    xy = norm(S(:,i) - S(:,j));
    if xy < s
      s = xy;
    end
  end
  if s < l
    l = s;
  end

  if s > r
    r = s;
  end
end

