function dispdist(d, h, layers, filename)
clf
V = d(2:end-1, 2:end-1, 2:end-1); % shave off boudaries
% Generate the isosurface from data in V 

% Construct domain matrices
x = h:h:1-h; 
y = x; z = x;
z = h:h:0.5;
[X, Y, Z] = meshgrid(x,y,z);
axis(gca, [0 1 0 1 0 1]);     % set axis limits
set(gca, 'PlotBoxAspectRatioMode', 'manual', 'PlotBoxAspectRatio', [1 1 1]);
hold on

% Draw isosurfaces at different distances
n = length(layers);
for i = 1:n
  p(i) = patch(isosurface(X, Y, Z, V(1:length(x), 1:length(y), 1:length(z)), layers(i)));
  isonormals(X,Y,Z,V(1:length(x), 1:length(y), 1:length(z)),p(i)); % Generate normals for viewing

  % Surface colors: Red -> farthest; Blue -> closest
  t = (i-1)/(n-1);
  c = [max(0, 2*t - 1), 1 - 2*abs(t - 0.5), max(0,1-2*t)];
  set(p(i), 'FaceColor', c, 'EdgeColor', 'none');
end

view(3);         % standard viewing angle
camlight         % light source from the camera
lighting gouraud % set shading method

if nargin > 3
  savesqpdf(filename); % optionally  save plot to a file
end

% Create the slider uicontrol with the nested function clipsurf as the callback
% in order to clip the generated surfaces above a plane at some threshold.
% this helps to view the internal structure of the contours of the distance function
sliderHandle = uicontrol('Style', 'slider', 'Callback', @clipsurf, ...
  'Units', 'normalized', 'Position', [0.85 0.15 0.1 0.7], ...
  'Min', 0, 'Max', 1, 'Value', 1);

  function clipsurf(theslider, eventdata)
    % Determine where the cutoff point is
    cutoff = get(theslider, 'Value');

    % Take the surface data below cutoff
    z = h:h:cutoff;
    V2 = V(1:length(x), 1:length(y), 1:length(z));

    % Set the current contour plot to be V2
    [X, Y, Z] = meshgrid(x,y,z);

    for i = 1:n
      [f,v] = isosurface(X, Y, Z, V2, layers(i));
      set(p(i), 'Vertices', v, 'Faces', f);
      isonormals(X,Y,Z,V2,p(i));
    end

  end % function clipsurf
end % function dispoutput
