function d = dist(S, N, randomsweeps, cycles)
% function d = dist(S, N)
% Compute the distance function from the given data set S
% on an NxNxN grid of [0,1]^3
%   randomsweeps - number of final random sweeps to do at end of each cycle
%   cycles       - number sweep cycles to perform

nc = N-1; % number of cells in each dimension
h = 1.0/double(nc);
% note that h = 1/(N-1) so we will multiply by nc when possible instead of
% dividing by h since multiplications are cheaper

static = logical(zeros(N,N,N));
d = sqrt(3)*ones(size(static)); % allocate space and initialize to a large value

for pt = S
  for i = 0:7
    % find bottommost leftmost closest corner to pt
    blc = [floor(pt(1)*nc)+1, floor(pt(2)*nc)+1 , floor(pt(3)*nc)+1];
    corvec = blc + de2bi(i,3); % each neighbour
    corcell = num2cell(corvec);
    static(corcell{:}) = 1;

    dst = norm(pt - (corvec' - ones(3,1))*h);
    if d(corcell{:}) > dst
      d(corcell{:}) = dst;
    end
  end
end % for each data point in S

if nargin < 3
  d = godunov(d, static, h);
elseif nargin < 4
  d = godunov(d, static, h, randomsweeps);
else
  d = godunov(d, static, h, randomsweeps, cycles);
end
