function v = gensphere(d, c, R)
% A function that computes the coordinates of points on a sphere with radius R,
% the generated sphere has uniform sampling density
%   d - desired distance between two neighbouring points
%   c - coordinates of the centre of the sphere
%   R - radius of the sphere
% NOTE: the north and south pole are always included, and should not be
% part of nlat.

if R == 0 % special case, a single point
  v = [c(1); c(2); c(3)];
  return;
end

n = round(pi*R/d) - 1;
dp = pi/(n+1); % single division in latitude angle

v = zeros(3, 2+n*n); % allocate space for coordinates
v(:,1) = c + [0; 0; R];  % north pole
v(:,2) = c + [0; 0; -R]; % south pole
count = 2;
for i = 1:n
  p = dp*i;
  sinp = sin(p);
  r = R*sinp;
  nlong = round(2*sinp*(n+1));
  dt = 2*pi/nlong;  % single division in longitude angle
  t = dt*[1:nlong];
  v(:,count+1:count+nlong) = c*ones(1,nlong) + [r*cos(t); r*sin(t); R*cos(p)*ones(1,nlong)];
  count = count + nlong;
end

v(:,count+1:end) = [];

