function u = godunov(u, static, h, randomsweeps, cycles)
% Solve the signed distance function PDE using Godunov's Scheme
%            u - initial function
%       static - a mask of the same size as u giving elements that should remain unchanged
%            N - size of u should be NxNxN
% randomsweeps - number of random sweeps to perform after the main 8 (default = 1)
%       cycles - number of cycles of sweeps to perform (default = 1)

[xn yn zn] = size(u);

% do 8 different sweeps (different orderings)
sweep = zeros(3, xn, 9);
sweep(:,:,1) = [1:xn;    1:yn;    1:zn];
sweep(:,:,2) = [xn:-1:1; 1:yn;    1:zn];
sweep(:,:,3) = [1:xn;    yn:-1:1; 1:zn];
sweep(:,:,4) = [1:xn;    1:yn;    zn:-1:1];
sweep(:,:,5) = [xn:-1:1; yn:-1:1; 1:zn];
sweep(:,:,6) = [1:xn;    yn:-1:1; zn:-1:1];
sweep(:,:,7) = [xn:-1:1; 1:yn;    zn:-1:1];
sweep(:,:,8) = [xn:-1:1; yn:-1:1; zn:-1:1];

if nargin < 4
  randomsweeps = 1;
end

if nargin < 5
  cycles = 1;
end

for i = 1:randomsweeps
  sweep(:,:,8+i) = sweep(:,:,1+floor(8*rand));
end

for c = 1:cycles
  for s = 1:8+randomsweeps
    for i = sweep(1,:,s)
      for j = sweep(2,:,s)
        for k = sweep(3,:,s)
          if static(i,j,k)
            continue;
          end
          if i < 2
            x = u(i+1,j,k);
          elseif i > xn-1
            x = u(i-1,j,k);
          else
            x = min(u(i-1,j,k), u(i+1,j,k));
          end
          if j < 2
            y = u(i,j+1,k);
          elseif j > yn-1
            y = u(i,j-1,k);
          else
            y = min(u(i,j-1,k), u(i,j+1,k));
          end
          if k < 2
            z = u(i,j,k+1);
          elseif k > zn-1
            z = u(i,j,k-1);
          else
            z = min(u(i,j,k-1), u(i,j,k+1));
          end
          a = sort([x,y,z]);
          h2 = h*h;
          t = a(1) + h;
          if t <= a(2)
            unew = t;
          else
            adiff = a(2) - a(1);
            t = 0.5*(a(1) + a(2) + sqrt(2*h2 - adiff*adiff));
            if t <= a(3)
              unew = t;
            else
              t = a(2) + h;
              if t <= a(3)
                unew = t;
              else
                adiff = a(3) - a(2);
                unew = 0.5*(a(2) + a(3) + sqrt(2*h2 - adiff*adiff));
              end % if t <= a(3)
            end % if t <= a(3)
          end % if t <= a(2)
          u(i,j,k) = min(u(i,j,k),unew);
        end % for k
      end % for j
    end % for i
  end % for sweeps
end % for cycles
