function [gu ngu meangu] = gradnarrow(u, V, mask, idx)
% compute gradient of the given function u and its norm in the narrow band
% u - the implicit function
% V - velocity vector, i.e. v_x = V(1,i,j,k) for instance
% mask and idx define the narrow band

[xn yn zn] = size(u);

initderivatives % initialize the derivative functions

gu = zeros(3, xn, yn, zn);
ngu = zeros(xn, yn, zn);
for l = idx
  i = l(1); j = l(2); k = l(3);
  if mask(i,j,k) == 2
    if V(1,i,j,k) > 0
      if dxxc(u,i-1,j,k) < dxxc(u,i,j,k)
        dx = dxmm(u,i,j,k);
      else
        dx = dxmp(u,i,j,k);
      end
    else
      if dxxc(u,i,j,k) < dxxc(u,i+1,j,k)
        dx = dxpm(u,i,j,k);
      else
        dx = dxpp(u,i,j,k);
      end
    end

    if V(2,i,j,k) > 0
      if dyyc(u,i,j-1,k) < dyyc(u,i,j,k)
        dy = dymm(u,i,j,k);
      else
        dy = dymp(u,i,j,k);
      end
    else
      if dyyc(u,i,j,k) < dyyc(u,i,j+1,k)
        dy = dypm(u,i,j,k);
      else
        dy = dypp(u,i,j,k);
      end
    end

    if V(3,i,j,k) > 0
      if dzzc(u,i,j,k-1) < dzzc(u,i,j,k)
        dz = dzmm(u,i,j,k);
      else
        dz = dzmp(u,i,j,k);
      end
    else
      if dzzc(u,i,j,k) < dzzc(u,i,j,k+1)
        dz = dzpm(u,i,j,k);
      else
        dz = dzpp(u,i,j,k);
      end
    end
    dxyz = [dx; dy; dz];
    gu(:,i,j,k) = dxyz;
    ngu(i,j,k) = norm(dxyz);
  end % if mask == 2
end % for each i

meangu = mean(mean(mean(ngu(mask==2))));
