% This script initializes all partial derivative discretizations
% using anonymous functions

% central differencing first-order partials
Dx = @(u,i,j,k) 0.5*(u(i+1,j,k) - u(i-1,j,k))*(size(u,1)+1);
Dy = @(u,i,j,k) 0.5*(u(i,j+1,k) - u(i,j-1,k))*(size(u,2)+1);
Dz = @(u,i,j,k) 0.5*(u(i,j,k+1) - u(i,j,k-1))*(size(u,3)+1);
nD = @(u,i,j,k) norm([Dx(u,i,j,k); Dy(u,i,j,k); Dz(u,i,j,k)]);
nD2 = @(u,i,j,k,m) Dx(u,i,j,k)*Dx(u,i,j,k) + Dy(u,i,j,k)*Dy(u,i,j,k) + Dz(u,i,j,k)*Dz(u,i,j,k);

% forward differencing first-order partials
Dxf = @(u,i,j,k) (u(i+1,j,k) - u(i,j,k))*(size(u,1)+1);
Dyf = @(u,i,j,k) (u(i,j+1,k) - u(i,j,k))*(size(u,2)+1);
Dzf = @(u,i,j,k) (u(i,j,k+1) - u(i,j,k))*(size(u,3)+1);

% backward differencing first-order partials
Dxb = @(u,i,j,k) Dxf(u,i-1,j,k);
Dyb = @(u,i,j,k) Dyf(u,i,j-1,k);
Dzb = @(u,i,j,k) Dzf(u,i,j,k-1);

% second order partials
Dxx = @(u,i,j,k) (u(i+1,j,k) - 2*u(i,j,k) + u(i-1,j,k))*(size(u,1)+1)*(size(u,1)+1);
Dyy = @(u,i,j,k) (u(i,j+1,k) - 2*u(i,j,k) + u(i,j-1,k))*(size(u,2)+1)*(size(u,2)+1);
Dzz = @(u,i,j,k) (u(i,j,k+1) - 2*u(i,j,k) + u(i,j,k-1))*(size(u,3)+1)*(size(u,3)+1);

Dxy = @(u,i,j,k) 0.25*(u(i+1,j+1,k)-u(i-1,j+1,k)-u(i+1,j-1,k)+u(i-1,j-1,k))*(size(u,1)+1)*(size(u,2)+1);
Dyz = @(u,i,j,k) 0.25*(u(i,j+1,k+1)-u(i,j-1,k+1)-u(i,j+1,k-1)+u(i,j-1,k-1))*(size(u,2)+1)*(size(u,3)+1);
Dzx = @(u,i,j,k) 0.25*(u(i+1,j,k+1)-u(i-1,j,k+1)-u(i+1,j,k-1)+u(i-1,j,k-1))*(size(u,3)+1)*(size(u,1)+1);
D = {nD2, nD, Dx, Dy, Dz, Dxx, Dyy, Dzz, Dxy, Dyz, Dzx};

% add to standard derivatives for the ENO scheme
dxpp = @(u,i,j,k) 0.5*(u(i+2,j,k) - u(i,j,k))*(size(u,1)+1);
dypp = @(u,i,j,k) 0.5*(u(i,j+2,k) - u(i,j,k))*(size(u,2)+1);
dzpp = @(u,i,j,k) 0.5*(u(i,j,k+2) - u(i,j,k))*(size(u,3)+1);

dxpm = @(u,i,j,k) 0.5*(3*u(i+1,j,k) - 4*u(i,j,k) + u(i-1,j,k))*(size(u,1)+1);
dypm = @(u,i,j,k) 0.5*(3*u(i,j+1,k) - 4*u(i,j,k) + u(i,j-1,k))*(size(u,2)+1);
dzpm = @(u,i,j,k) 0.5*(3*u(i,j,k+1) - 4*u(i,j,k) + u(i,j,k-1))*(size(u,3)+1);

dxmp = @(u,i,j,k) Dx(u,i,j,k);
dymp = @(u,i,j,k) Dy(u,i,j,k);
dzmp = @(u,i,j,k) Dz(u,i,j,k);

dxmm = @(u,i,j,k) 0.5*(3*u(i,j,k) - 4*u(i-1,j,k) + u(i-2,j,k))*(size(u,1)+1);
dymm = @(u,i,j,k) 0.5*(3*u(i,j,k) - 4*u(i,j-1,k) + u(i,j-2,k))*(size(u,2)+1);
dzmm = @(u,i,j,k) 0.5*(3*u(i,j,k) - 4*u(i,j,k-1) + u(i,j,k-2))*(size(u,3)+1);

dxxc = @(u,i,j,k) abs(u(i+1,j,k) - 2*u(i,j,k) + u(i-1,j,k));
dyyc = @(u,i,j,k) abs(u(i,j+1,k) - 2*u(i,j,k) + u(i,j-1,k));
dzzc = @(u,i,j,k) abs(u(i,j,k+1) - 2*u(i,j,k) + u(i,j,k-1));
