function u0 = initsurf(d, eps, h)
% construct the initial surface

[xn yn zn] = size(d);

discovered = logical(zeros(size(d))); % used for DFS

stack = zeros(3,xn*yn*zn);

stack(:,1) = [1 1 1];
sn = 1; % stack size

while sn > 0
  % pop an index that is labelled exterior
  s = stack(:,sn);
  i = s(1); j = s(2); k = s(3);
  sn = sn - 1;
  discovered(i,j,k) = 1;
  if d(i,j,k) > eps
    if i < xn && ~discovered(i+1,j,k)
      sn = sn + 1;
      stack(:,sn) = [i+1, j, k];
    end
    if j < yn && ~discovered(i,j+1,k)
      sn = sn + 1;
      stack(:,sn) = [i, j+1, k];
    end
    if k < zn && ~discovered(i,j,k+1)
      sn = sn + 1;
      stack(:,sn) = [i, j, k+1];
    end

    if i > 1 && ~discovered(i-1,j,k)
      sn = sn + 1;
      stack(:,sn) = [i-1, j, k];
    end
    if j > 1 && ~discovered(i,j-1,k)
      sn = sn + 1;
      stack(:,sn) = [i, j-1, k];
    end
    if k > 1 && ~discovered(i,j,k-1)
      sn = sn + 1;
      stack(:,sn) = [i, j, k-1];
    end
  end
end % while

u0 = zeros(size(d)); % allocate space for initial surface
u0(discovered) = d(discovered) - eps;
u0(~discovered) = -sqrt(3);
u0 = -godunov(-u0, discovered, h);

end % function initsurf
