% Construct a vector of input points

filename = 'tori';
fprintf('Constructing sample...\n');
% First consider a simple parametrized sphere
%sep = 0.2; % separation between points
%R = 0.1; % sphere radius
%sep = 0.05; % separation between points
%R = 0.2; % sphere radius
%c = [0.5; 0.5; 0.5];              % centre of the sphere
%S = gensphere(sep, c, R); % set of data points
S = tori;
fprintf('Sample size = %d\n', size(S,2));
dispinput(S, [filename 'in']);            % save and display the data

% Construct initial surface
N = 49; % number of gridpoints
%N = 33; % number of gridpoints
h = 1/(N-1);
x = 0:h:1;
n = length(x);

% get the minimum local feature size and maximum distance 
% between connected points in S, to get a good epsilon estimate
fprintf('Getting data information...\n');
[l r] = datainfo(S); 
if l - 2*h < -0.15
  fprintf('WARNING: Resolution too SMALL to resolve local features.\n');
end
if l - 2*h > 0
  fprintf('WARNING: Resolution too LARGE for distance computation.\n');
end

if 0.5*r - l > 0.01
  fprintf('WARNING: Non-uniform sampling density.\n');
end

fprintf('Computing the distance function...\n');
% compute the distance function
d = dist(S, N, 1, 1);

% compute initial surface
% set epsilon to resolve all local features, and maintain connectivity
eps = max([0.99*l 0.501*r]);
eps = 0.06;
fprintf('Epsilon = %f\n', eps);
fprintf('Constructing initial surface...\n');
u0 = initsurf(d, eps, h);
figure();
dispoutput(u0, h, [filename '0']);

fprintf('Evolving the surface...\n');
u = propagate(u0, d, h, 1e-4, 105, filename);
